# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 - 2021 TU Wien.
#
# Invenio-Config-TUW is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Pytest configuration.

See https://pytest-invenio.readthedocs.io/ for documentation on which test
fixtures are available.
"""

import shutil
import tempfile

import pytest
from flask import Flask
from flask_babelex import Babel

from invenio_config_tuw import InvenioConfigTUW


@pytest.fixture(scope="module")
def celery_config():
    """Override pytest-invenio fixture.

    TODO: Remove this fixture if you add Celery support.
    """
    return {}


@pytest.fixture(scope="module")
def create_app(instance_path):
    """Application factory fixture."""

    def factory(**config):
        app = Flask("testapp", instance_path=instance_path)
        app.config.update(**config)
        Babel(app)
        InvenioConfigTUW(app)
        return app

    return factory



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 - 2021 TU Wien.
#
# Invenio-Config-TUW is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Invenio module containing some customizations and configuration for TU Wien."""

from flask_security.signals import user_registered

from . import config
from .auth.utils import auto_trust_user
from .permissions import TUWRecordPermissionPolicy


@user_registered.connect
def auto_trust_new_user(sender, user, **kwargs):
    # NOTE: 'sender' and 'kwargs' are ignored, but they're required to match the
    #       expected function signature
    # NOTE: this function won't be called when a user is created via the CLI
    #       ('invenio users create'), because it doesn't send the 'user_registered'
    #       signal
    auto_trust_user(user)


class InvenioConfigTUW(object):
    """Invenio-Config-TUW extension."""

    def __init__(self, app=None):
        """Extension initialization."""
        if app:
            self.init_app(app)

    def init_app(self, app):
        """Flask application initialization."""
        self.init_config(app)

        # override the permission policies
        svc = app.extensions["invenio-rdm-records"].records_service
        svc.config.permission_policy_cls = TUWRecordPermissionPolicy
        svc.files.config.permission_policy_cls = TUWRecordPermissionPolicy
        svc.draft_files.config.permission_policy_cls = TUWRecordPermissionPolicy
        svc.secret_links.config.permission_policy_cls = TUWRecordPermissionPolicy

        app.extensions["invenio-config-tuw"] = self

    def init_config(self, app):
        """Initialize configuration."""
        # Use theme's base template if theme is installed
        for k in dir(config):
            if len(k.replace("_", "")) >= 3 and k.isupper():
                app.config.setdefault(k, getattr(config, k))



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 - 2021 TU Wien.
#
# Invenio-Config-TUW is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Utility functions for authentication."""


from flask import current_app
from invenio_accounts.proxies import current_datastore
from invenio_db import db

from ..utils import get_user_by_username


def create_username_from_info(user_info):
    """Create a unique username from the specified user info."""
    email = user_info["email"]
    raw_username = username = email[: email.index("@")].replace(".", "-")
    num = 0

    if get_user_by_username(username) is not None:
        num += 1
        username = "{}{}".format(raw_username, num)

    return username


def auto_trust_user(user):
    """Automatically trust newly registered users if that's configured."""
    auto_trust_enabled = current_app.config.get("CONFIG_TUW_AUTO_TRUST_USERS")
    trust_check = current_app.config.get("CONFIG_TUW_AUTO_TRUST_CONDITION", None)

    if user and auto_trust_enabled:
        # if the user was created successfully and auto-trust is enabled...
        role = current_datastore.find_role("trusted-user")
        if not role:
            role_data = {
                "name": "trusted-user",
                "description": "Users trusted with extra permissions",
            }
            role = current_datastore.create_role(**role_data)
            db.session.commit()

        # if no trust condition is specified, don't trust the user
        if trust_check is None or trust_check(user):
            # NOTE: the add_role_to_user function is idempotent
            current_datastore.add_role_to_user(user, role)
            db.session.commit()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 - 2021 TU Wien.
#
# Invenio-Config-TUW is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Invenio module containing some customizations and configuration for TU Wien."""

from .config import TUWSSOSettingsHelper
from .handlers import authorized_signup_handler, info_handler, signup_handler

__all__ = (
    "authorized_signup_handler",
    "info_handler",
    "signup_handler",
    "TUWSSOSettingsHelper",
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 - 2021 TU Wien.
#
# Invenio-Config-TUW is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Version information for Invenio-Config-TUW.

This file is imported by ``invenio_config_tuw.__init__``,
and parsed by ``setup.py``.
"""

__version__ = "2021.1"



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 - 2021 TU Wien.
#
# Invenio-Config-TUW is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Utility functions for authentication."""


from invenio_oauthclient.contrib.keycloak import KeycloakSettingsHelper


class TUWSSOSettingsHelper(KeycloakSettingsHelper):
    """KeycloakSettingsHelper, adjusted for the needs of TU Data."""

    def get_handlers(self):
        return {
            "authorized_handler": "invenio_config_tuw.auth:authorized_signup_handler",
            "disconnect_handlerdler": (
                "invenio_oauthclient.contrib.keycloak.handlers:disconnect_handler"
            ),
            "signup_handler": {
                "info": "invenio_config_tuw.auth:info_handler",
                "setup": "invenio_oauthclient.contrib.keycloak.handlers:setup_handler",
                "view": "invenio_config_tuw.auth:signup_handler",
            },
        }



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 - 2021 TU Wien.
#
# Invenio-Config-TUW is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Custom handlers for the Keycloak integration."""


from flask import current_app, redirect, render_template, request, session, url_for
from flask_login import current_user
from invenio_db import db
from invenio_oauthclient.contrib.keycloak.helpers import get_user_info
from invenio_oauthclient.errors import (
    OAuthClientMustRedirectSignup,
    OAuthClientUnAuthorized,
)
from invenio_oauthclient.handlers.base import base_signup_handler
from invenio_oauthclient.handlers.ui import (
    oauth_remote_error_handler,
    oauth_resp_remote_error_handler,
)
from invenio_oauthclient.handlers.utils import (
    get_session_next_url,
    response_token_setter,
    token_getter,
    token_session_key,
)
from invenio_oauthclient.proxies import current_oauthclient
from invenio_oauthclient.signals import (
    account_info_received,
    account_setup_committed,
    account_setup_received,
)
from invenio_oauthclient.utils import (
    create_csrf_disabled_registrationform,
    create_registrationform,
    fill_form,
    oauth_authenticate,
    oauth_get_user,
    oauth_register,
)
from invenio_userprofiles.models import UserProfile

from .utils import auto_trust_user, create_username_from_info, get_user_by_username


def info_handler(remote, resp):
    """Retrieve remote account information for finding matching local users."""
    # NOTE: for lookup of local users, only the 'external_id' and 'user.email' are used
    #       => calculation of the 'user.profile.username' here should actually be safe
    user_info = get_user_info(remote, resp)

    # fill out the information required by
    # 'invenio-accounts' and 'invenio-userprofiles'.
    #
    # note: "external_id": `preferred_username` should also work,
    #       as it is seemingly not editable in Keycloak
    result = {
        "user": {
            "active": True,
            "email": user_info["email"],
            "profile": {
                "full_name": user_info.get("name"),
                "username": user_info.get("preferred_username"),
            },
        },
        "external_id": user_info["sub"],
        "external_method": remote.name,
    }

    return result


@oauth_remote_error_handler
def signup_handler(remote, *args, **kwargs):
    """Handle extra signup information.

    :param remote: The remote application.
    :returns: Redirect response or the template rendered.
    """
    try:
        form = create_registrationform(request.form)
        next_url = base_signup_handler(remote, form, *args, **kwargs)
        if form.is_submitted():
            if next_url:
                return redirect(next_url)
            else:
                return redirect("/")

        return render_template(
            current_app.config["OAUTHCLIENT_SIGNUP_TEMPLATE"],
            form=form,
            remote=remote,
            app_title=current_app.config["OAUTHCLIENT_REMOTE_APPS"][remote.name].get(
                "title", ""
            ),
            app_description=current_app.config["OAUTHCLIENT_REMOTE_APPS"][
                remote.name
            ].get("description", ""),
            app_icon=current_app.config["OAUTHCLIENT_REMOTE_APPS"][remote.name].get(
                "icon", None
            ),
        )
    except OAuthClientUnAuthorized:
        # Redirect the user after registration (which doesn't include the
        # activation), waiting for user to confirm his email.
        return redirect(url_for("security.login"))


@oauth_resp_remote_error_handler
def authorized_signup_handler(resp, remote, *args, **kwargs):
    """Handle sign-in/up functionality.

    :param remote: The remote application.
    :param resp: The response.
    :returns: Redirect response.
    """
    next_url = base_authorized_signup_handler(resp, remote, *args, **kwargs)
    # Redirect to next
    if next_url:
        return redirect(next_url)

    return redirect(url_for("invenio_oauthclient_settings.index"))


def base_authorized_signup_handler(resp, remote, *args, **kwargs):
    """Handle sign-in/up functionality.

    :param remote: The remote application.
    :param resp: The response.
    :returns: Redirect response.
    """
    # Remove any previously stored auto register session key
    session.pop(token_session_key(remote.name) + "_autoregister", None)

    # Store token in session
    # ----------------------
    # Set token in session - token object only returned if
    # current_user.is_autenticated().
    token = response_token_setter(remote, resp)
    handlers = current_oauthclient.signup_handlers[remote.name]

    # Sign-in/up user
    # ---------------
    if not current_user.is_authenticated:
        account_info = handlers["info"](resp)
        account_info_received.send(
            remote, token=token, response=resp, account_info=account_info
        )

        user = oauth_get_user(
            remote.consumer_key,
            account_info=account_info,
            access_token=token_getter(remote)[0],
        )
        user_found = user is not None

        if not user_found:
            # Auto sign-up if user not found
            user_info = account_info["user"]
            username = user_info["profile"]["username"]
            form = create_csrf_disabled_registrationform(remote)

            form = fill_form(form, account_info["user"])
            if not form.validate() or get_user_by_username(username) is not None:
                # if the 'preferred_username' wasn't valid or already taken,
                # try to auto-generate a valid and unique username
                account_info["user"]["profile"]["username"] = create_username_from_info(
                    user_info
                )
                form = fill_form(form, account_info["user"])

            remote_apps = current_app.config["OAUTHCLIENT_REMOTE_APPS"]
            precedence_mask = remote_apps[remote.name].get("precedence_mask")
            user = oauth_register(form, account_info["user"], precedence_mask)

            # if registration fails ...
            if user is None:
                # requires extra information
                session[token_session_key(remote.name) + "_autoregister"] = True
                session[token_session_key(remote.name) + "_account_info"] = account_info
                session[token_session_key(remote.name) + "_response"] = resp
                db.session.commit()
                raise OAuthClientMustRedirectSignup()

            else:
                # auto-trust new users, if they meet the configured criteria
                # NOTE: this was moved here from ext.py (as a handler for the
                # flask_security.signals.user_registered signal), because that
                # didn't quite work with a containerized setup...
                auto_trust_user(user)

        # Authenticate user
        if not oauth_authenticate(
            remote.consumer_key, user, require_existing_link=False
        ):
            raise OAuthClientUnAuthorized()

        if user_found:
            # the user already exists: update their details!
            user_info = account_info.get("user", {})
            new_email = user_info.get("email", user.email)
            new_profile = user_info.get("profile", {})
            new_full_name = new_profile.get("full_name")

            if user.email != new_email:
                user.email = new_email

                # our usernames are based on the users' email addresses;
                # thus, we need to update the username if the email address changes
                if user.profile:
                    user.profile.username = create_username_from_info(user_info)

            if new_profile and user.profile and new_full_name != user.profile.full_name:
                user.profile.full_name = new_full_name

        # Link account
        # ------------
        # Need to store token in database instead of only the session when
        # called first time.
        token = response_token_setter(remote, resp)

    # Setup account
    # -------------
    if not token.remote_account.extra_data:
        account_setup = handlers["setup"](token, resp)
        account_setup_received.send(
            remote, token=token, response=resp, account_setup=account_setup
        )
        db.session.commit()
        account_setup_committed.send(remote, token=token)
    else:
        db.session.commit()

    # Redirect to next
    next_url = get_session_next_url(remote.name)
    if next_url:
        return next_url



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 - 2021 TU Wien.
#
# Invenio-Config-TUW is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Invenio module containing some customizations and configuration for TU Wien."""

from .ext import InvenioConfigTUW
from .version import __version__

__all__ = ("__version__", "InvenioConfigTUW")



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 - 2021 TU Wien.
#
# Invenio-Config-TUW is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

from flask import Blueprint

blueprint = Blueprint(
    "invenio_config_tuw",
    __name__,
    template_folder="templates",
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 - 2021 TU Wien.
#
# Invenio-Config-TUW is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Invenio module containing some customizations and configuration for TU Wien."""

import os

from setuptools import find_packages, setup

readme = open("README.rst").read()
history = open("CHANGES.rst").read()

tests_require = [
    "check-manifest>=0.25",
    "coverage>=4.0",
    "isort>=4.3.3",
    "pydocstyle>=2.0.0",
    "pytest-cov>=2.5.1",
    "pytest-pep8>=1.0.6",
    "pytest-invenio>=1.2.1",
]

extras_require = {
    "docs": [
        "Sphinx>=1.5.1",
    ],
    "tests": tests_require,
}

extras_require["all"] = []
for reqs in extras_require.values():
    extras_require["all"].extend(reqs)

setup_requires = [
    "Babel>=1.3",
    "pytest-runner>=3.0.0,<5",
]

install_requires = [
    "Flask-BabelEx>=0.9.4",
    "invenio-config>=1.0.3",
]

packages = find_packages()


# Get the version string. Cannot be done with import!
g = {}
with open(os.path.join("invenio_config_tuw", "version.py"), "rt") as fp:
    exec(fp.read(), g)
    version = g["__version__"]

setup(
    name="invenio-config-tuw",
    version=version,
    description=__doc__,
    long_description=readme + "\n\n" + history,
    keywords="invenio tu wien configuration",
    license="MIT",
    author="TU Wien",
    author_email="tudata@tuwien.ac.at",
    url="https://gitlab.tuwien.ac.at/fairdata/invenio-config-tuw",
    packages=packages,
    zip_safe=False,
    include_package_data=True,
    platforms="any",
    entry_points={
        "invenio_base.apps": [
            "invenio_config_tuw = invenio_config_tuw:InvenioConfigTUW",
        ],
        "invenio_base.api_apps": [
            "invenio_config_tuw = invenio_config_tuw:InvenioConfigTUW",
        ],
        "invenio_base.blueprints": [
            "invenio_config_tuw = invenio_config_tuw.views:blueprint",
        ],
        "invenio_config.module": [
            # configs are loaded by invenio-config in lexicographic order
            # of their entrypoint names, and we want to override the other
            # configs here -- while not overriding invenio.cfg (which could
            # be done in the ext.init_config(app) method)
            "zzz_invenio_config_tuw = invenio_config_tuw.config",
        ],
    },
    extras_require=extras_require,
    install_requires=install_requires,
    setup_requires=setup_requires,
    tests_require=tests_require,
    classifiers=[
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Development Status :: 4 - Beta",
    ],
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 - 2021 TU Wien.
#
# Invenio-Config-TUW is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

from flask_principal import RoleNeed
from invenio_rdm_records.services import RDMRecordPermissionPolicy
from invenio_rdm_records.services.generators import (
    IfDraft,
    IfRestricted,
    RecordOwners,
    SecretLinks,
)
from invenio_records_permissions.generators import (
    Admin,
    AnyUser,
    Disable,
    Generator,
    SuperUser,
    SystemProcess,
)


class TrustedUsers(Generator):
    """Allows users with the "trusted-user" role."""

    def needs(self, record=None, **kwargs):
        """Enabling Needs."""
        return [RoleNeed("trusted-user")]


secret_links = {
    "edit": [SecretLinks("edit")],
    "view": [SecretLinks("edit"), SecretLinks("view")],
    "view_record": [SecretLinks("edit"), SecretLinks("view"), SecretLinks("record")],
    "view_files": [SecretLinks("edit"), SecretLinks("view"), SecretLinks("files")],
    "preview": [SecretLinks("edit"), SecretLinks("preview")],
}

owner_if_restricted_record = IfRestricted(
    "record", then_=[RecordOwners()], else_=[AnyUser()]
)

owner_if_restricted_files = IfRestricted(
    "files", then_=[RecordOwners()], else_=[AnyUser()]
)


class TUWRecordPermissionPolicy(RDMRecordPermissionPolicy):
    # note: edit := create a draft from a record (i.e. putting it in edit mode)
    # fmt: off
    # high-level permissions
    can_manage             = [RecordOwners(),             Admin(), SuperUser(), SystemProcess()]                                # noqa
    can_curate             = [RecordOwners(),             Admin(), SuperUser(), SystemProcess()] + secret_links["edit"]         # noqa
    can_preview            = [RecordOwners(),             Admin(), SuperUser(), SystemProcess()] + secret_links["preview"]      # noqa
    can_view               = [RecordOwners(),             Admin(), SuperUser(), SystemProcess()] + secret_links["view"]         # noqa

    # records
    can_search             = [AnyUser(),                  Admin(), SuperUser(), SystemProcess()]                                # noqa
    can_read               = [owner_if_restricted_record, Admin(), SuperUser(), SystemProcess()] + secret_links["view_record"]  # noqa
    can_read_files         = [owner_if_restricted_files,  Admin(), SuperUser(), SystemProcess()] + secret_links["view_files"]   # noqa
    can_create             = [TrustedUsers(),             Admin(), SuperUser(), SystemProcess()]                                # noqa

    # drafts
    can_search_drafts      = [TrustedUsers(),             Admin(), SuperUser(), SystemProcess()]                                # noqa
    can_read_draft         = [                          *can_preview                           ]                                # noqa
    can_update_draft       = [                          *can_curate                            ]                                # noqa
    can_draft_read_files   = [                          *can_preview                           ]                                # noqa
    can_draft_create_files = [                          *can_curate                            ]                                # noqa
    can_draft_update_files = [                          *can_curate                            ]                                # noqa
    can_draft_delete_files = [                          *can_curate                            ]                                # noqa

    # PIDs
    can_pid_reserve        = [                          *can_curate                            ]                                # noqa
    can_pid_delete         = [                          *can_curate                            ]                                # noqa

    # actions
    can_edit               = [                          *can_curate                            ]                                # noqa
    can_delete_draft       = [                          *can_curate                            ]                                # noqa
    can_new_version        = [                          *can_curate                            ]                                # noqa
    can_publish            = [                          *can_curate                            ]                                # noqa
    can_lift_embargo       = [                          *can_manage                            ]                                # noqa

    # disabled (record management in InvenioRDM goes through drafts)
    can_update             = [                           Disable()                             ]                                # noqa
    can_delete             = [                           Disable()                             ]                                # noqa
    can_create_files       = [                           Disable()                             ]                                # noqa
    can_update_files       = [                           Disable()                             ]                                # noqa
    can_delete_files       = [                           Disable()                             ]                                # noqa
    # fmt: on



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 - 2021 TU Wien.
#
# Invenio-Config-TUW is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Utility functions."""

from flask_principal import Identity
from invenio_access import any_user
from invenio_access.utils import get_identity
from invenio_accounts import current_accounts
from invenio_userprofiles.models import UserProfile


def get_user_by_username(username):
    """Get the user identified by the username."""
    profile = UserProfile.query.filter(UserProfile.username == username).one_or_none()

    if profile is not None:
        return profile.user

    return None


def get_user(identifier):
    """Get the user identified by the given ID, email or username."""
    user = current_accounts.datastore.get_user(identifier)
    if user is None:
        get_user_by_username(identifier)

    return user


def get_identity_for_user(user):
    """Get the Identity for the user specified via email, ID or username."""
    identity = None
    if user is not None:
        # note: this seems like the canonical way to go
        #       'as_user' can be either an integer (id) or email address
        u = get_user(user)
        if u is not None:
            identity = get_identity(u)
        else:
            raise LookupError("user not found: %s" % user)

    if identity is None:
        identity = Identity(1)

    identity.provides.add(any_user)
    return identity



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 - 2021 TU Wien.
#
# Invenio-Config-TUW is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Invenio module containing some customizations and configuration for TU Wien."""

from datetime import datetime

from flask import current_app, url_for
from flask_babelex import gettext as _
from invenio_oauthclient.views.client import auto_redirect_login
from invenio_userprofiles.api import current_userprofile
from werkzeug.local import LocalProxy
from wtforms import BooleanField, validators

from .auth import TUWSSOSettingsHelper

_security = LocalProxy(lambda: current_app.extensions["security"])


def tuw_registration_form(*args, **kwargs):
    terms_of_use_file = _("TU_Data_Terms_of_Use_20210511_en.pdf")
    terms_of_use_url = url_for("static", filename=(f"documents/{terms_of_use_file}"))
    message = f"Accept the <a href='{terms_of_use_url}' target='_blank'>Terms and Conditions</a>"

    class MyRegistrationForm(_security.confirm_register_form):
        email = None
        profile = None
        password = None
        recaptcha = None
        submit = None  # defined in the template
        terms_of_use = BooleanField(message, [validators.required()])

    return MyRegistrationForm(*args, **kwargs)


def check_user_email_for_tuwien(user):
    """Check if the user's email belongs to TU Wien (but not as a student)."""
    domain = user.email.split("@")[-1]
    return domain.endswith("tuwien.ac.at") and "student" not in domain


def current_user_as_creator():
    """Use the currently logged-in user to populate a creator."""
    profile = current_userprofile
    if profile is None or profile.full_name is None:
        return []

    name_parts = profile.full_name.split()
    if len(name_parts) <= 1:
        return []

    first_name = " ".join(name_parts[:-1])
    last_name = name_parts[-1]
    full_name = "{}, {}".format(last_name, first_name)
    creator = {
        "affiliations": [
            {
                "identifiers": [{"identifier": "04d836q62", "scheme": "ror"}],
                "name": "TU Wien, Vienna, Austria",
            }
        ],
        "person_or_org": {
            "family_name": last_name,
            "given_name": first_name,
            "identifiers": [],
            "name": full_name,
            "type": "personal",
        },
    }

    return [creator]


# Invenio-Config-TUW
# ==================

CONFIG_TUW_AUTO_TRUST_USERS = True
"""Whether or not to auto-assign the 'trusted-user' role to new users."""

CONFIG_TUW_AUTO_TRUST_CONDITION = check_user_email_for_tuwien
"""Function for checking if the user is eligible for auto-trust.

This must be a function that accepts a 'user' argument and returns a boolean value.
Alternatively, it can be set to None. This is the same as ``lambda u: True``.
"""


# Invenio-Mail
# ============
# See https://invenio-mail.readthedocs.io/en/latest/configuration.html

MAIL_SERVER = "localhost"
"""Domain ip where mail server is running."""

SECURITY_EMAIL_SENDER = "no-reply@researchdata.tuwien.ac.at"
"""Email address used as sender of account registration emails."""

SECURITY_EMAIL_SUBJECT_REGISTER = _("Welcome to TU Data!")
"""Email subject for account registration emails."""

MAIL_SUPPRESS_SEND = True
"""Disable email sending by default."""


# Invenio-Previewer
# =================

PREVIEWER_MAX_IMAGE_SIZE_BYTES = 10 * 1024 * 1024  # 10 MB

PREVIEWER_MAX_FILE_SIZE_BYTES = 10 * 1024 * 1024  # 10 MB


# Authentication
# ==============

SECURITY_CHANGEABLE = False
"""Allow password change by users."""

SECURITY_RECOVERABLE = False
"""Allow password recovery by users."""

SECURITY_REGISTERABLE = False
""""Allow users to register."""

SECURITY_CONFIRMABLE = False
"""Allow user to confirm their email address."""

ACCOUNTS = True
"""Tells if the templates should use the accounts module."""

ACCOUNTS_LOCAL_LOGIN_ENABLED = False
"""Disable local login (rely only on OAuth)."""

USERPROFILES_READ_ONLY = True
"""Prevent users from updating their profiles."""


# Invenio-OAuthClient
# ===================

ACCOUNTS_LOGIN_VIEW_FUNCTION = auto_redirect_login

OAUTHCLIENT_SIGNUP_FORM = tuw_registration_form

OAUTHCLIENT_AUTO_REDIRECT_TO_EXTERNAL_LOGIN = True

helper = TUWSSOSettingsHelper(
    title="TU Wien SSO",
    description="TU Wien Single Sign-On",
    base_url="https://s194.dl.hpc.tuwien.ac.at",
    realm="tu-data-test",
)

OAUTHCLIENT_KEYCLOAK_REALM_URL = helper.realm_url
OAUTHCLIENT_KEYCLOAK_USER_INFO_URL = helper.user_info_url
OAUTHCLIENT_KEYCLOAK_AUD = "tu-data-test"
OAUTHCLIENT_KEYCLOAK_VERIFY_AUD = True

keycloak_remote_app = helper.remote_app
keycloak_remote_app["precedence_mask"] = {
    "email": True,
    "profile": {"username": True, "full_name": True},
}

OAUTHCLIENT_REMOTE_APPS = {
    "keycloak": keycloak_remote_app,
}


# Invenio-App-RDM
# ================

APP_RDM_DEPOSIT_FORM_DEFAULTS = {
    "publication_date": lambda: datetime.now().strftime("%Y-%m-%d"),
    "creators": current_user_as_creator,
    "rights": [
        {
            "id": "cc-by-4.0",
            "title": "Creative Commons Attribution 4.0 International",
            "description": (
                "The Creative Commons Attribution license allows "
                "re-distribution and re-use of a licensed work "
                "on the condition that the creator is "
                "appropriately credited."
            ),
            "link": "https://creativecommons.org/licenses/by/4.0/legalcode",
        }
    ],
    "publisher": "TU Wien",
    "resource_type": {
        "id": "dataset",
    },
}


# Misc. Configuration
# ===================

# Default locale (language)
BABEL_DEFAULT_LOCALE = "en"

# Default time zone
BABEL_DEFAULT_TIMEZONE = "Europe/Vienna"

#: Recaptcha public key (change to enable).
RECAPTCHA_PUBLIC_KEY = None

#: Recaptcha private key (change to enable).
RECAPTCHA_PRIVATE_KEY = None

RATELIMIT_ENABLED = False

PREFERRED_URL_SCHEME = "https"
